/** \file
 * Header for the CAtlBackground interface
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Guillaume Boulet-Gercourt
 * \version 1.0
 * \date 24/06/2004
 */

#ifndef CAtlBoundingBox_h
#define	CAtlBoundingBox_h

// Basic class.
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace AvAtl {


/// Bounding box 
/** \class CAtlBoundingBox 
 *
 * \author Abvent R&D Copyright (c) 2004
 * \author Guillaume Boulet-Gercourt
 * \version 1.0
 * \date 24/06/2004
 */
class CAtlBoundingBox
{
public:
	/// Constructor with min and max data
	CAtlBoundingBox(const CAtlVector& inMin, const CAtlVector& inMax) : mMin(inMin), mMax(inMax) {}

	/// Empty constructor
						CAtlBoundingBox() : mMin(1e30, 1e30, 1e30), mMax(-1e30, -1e30, -1e30) {}

	/// Destructor
						~CAtlBoundingBox() {}

	/// Extent the box to include the point
	void				addPoint(const CAtlVector& inPoint)
	{
		if (mMin.X() > inPoint.X())
			mMin.X(inPoint.X());
		if (mMin.Y() > inPoint.Y())
			mMin.Y(inPoint.Y());
		if (mMin.Z() > inPoint.Z())
			mMin.Z(inPoint.Z());

		if (mMax.X() < inPoint.X())
			mMax.X(inPoint.X());
		if (mMax.Y() < inPoint.Y())
			mMax.Y(inPoint.Y());
		if (mMax.Z() < inPoint.Z())
			mMax.Z(inPoint.Z());
	}

	/// Get the minimum point of the box
	const CAtlVector&	getMin() const {return mMin;}

	/// Set the minimum point of the box
	void				setMin(const CAtlVector& inMin) {mMin = inMin;}

	/// Get the maximum point of the box
	const CAtlVector&	getMax() const {return mMax;}

	/// Set the maximum point of the box
	void				setMax(const CAtlVector& inMax) {mMax = inMax;}

private:
	/// The minimum point of the box
	CAtlVector		mMin;

	/// The maximum point of the box
	CAtlVector		mMax;
};


} // namespace AvAtl


#endif // CAtlBoundingBox_h
