/** \file
 * Header for the class CAtlTriangle
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlTriangle_h
#define CAtlTriangle_h


// Basic class.
#ifndef CAtlVector_h
	#include "CAtlVector.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {

/// Contains the 3 points of a triangle.
/** \class CAtlTriangle 
 *	The class CAtlTriangle is a class use to describe a triangle in a 3d space.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlTriangle
{
public:
	/// Constructors
							CAtlTriangle() {}

	/// Constructors
							CAtlTriangle(const CAtlVector& inP0, const CAtlVector& inP1, const CAtlVector& inP2)
							:	mP0(inP0), mP1(inP1), mP2(inP2)
							{}

	/// Copy constructors
							CAtlTriangle(const CAtlTriangle& inOther)
							:	mP0(inOther.mP0), mP1(inOther.mP1), mP2(inOther.mP2)
							{}

	/// Access to the first point of the triangle.
	const CAtlVector&	p0() const { return mP0; }
	/// Access to the first point of the triangle.
	CAtlVector&			p0() { return mP0; }

	/// Access to the second point of the triangle.
	const CAtlVector&	p1() const { return mP1; }
	/// Access to the second point of the triangle.
	CAtlVector&			p1() { return mP1; }

	/// Access to the third point of the triangle.
	const CAtlVector&	p2() const { return mP2; }
	/// Access to the third point of the triangle.
	CAtlVector&			p2() { return mP2; }

	/// Return the point specified by the index.
	const CAtlVector&	operator[](unsigned int i) const { TestIndex(i);  return (&mP0)[i]; }

	/// Return the point specified by the index.
	CAtlVector&			operator[](unsigned int i) { TestIndex(i);  return (&mP0)[i]; }

private:
	/// Validate the index, throw if invalid
	static void			TestIndex(unsigned int i)
							{
								if (i > 2)
									throw std::out_of_range("CAtlTriangle::TestIndex");
							}

	/// first point of the triangle.
	CAtlVector			mP0;
	/// second point of the triangle.
	CAtlVector			mP1;
	/// thrid point of the triangle.
	CAtlVector			mP2;
};

}

#endif
