/*!
	\file	IAtlInterpolatedValue.h
	\date	04 08 05
	\author	Abvent R&D copyright (c) 2005
	\author	Guillaume Boulet-Gercourt
	Header for the IAtlInterpolatedValue interface.
*/
#ifndef		IAtlInterpolatedValue_h
# define	IAtlInterpolatedValue_h

// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif
#ifndef IAtlKeyFrame_h
	#include "IAtlKeyFrame.h"
#endif

namespace	AvAtl {

/// List of interpolated values.
typedef TAtlObjectMgr<class IAtlInterpolatedValue>		AtlInterpolatedValuesMgr;

/*!
	\interface	IAtlInterpolatedValue
	\date		04 08 05
	\author		Abvent R&D copyright (C) 2005
	\author		Guillaume Boulet-Gercourt
	\brief		Describes an interpolated value of an animated element for Artlantis Studio
*/
class	IAtlInterpolatedValue
{
public:
	/// Put an XML representation of the interpolated value inside the father node.
	/** \retval the new node of the interpolated value.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef				putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the interpolated value with the XML.
	/** \retval true initialisation from XML succeed.
		\retval false an error occurs. */
	virtual bool					initWithXML(XMLNodeRef inFatherNode) = 0;

	/// eValueType describe the type of the interpolated value. I can be the position of a camera, ...
	enum	eValueType
	{
		kUnknow = 0,

		// cameras specific
		kCameraPosition,		// value is a double. the parameter 't' of the bezier spline. 0 is the start of the spline, 1 is the end. 
								/** Warning : the default value of this interpolated value is an absolute 3d coordinate */
		kCameraTarget,			// 3D Coordinates (CAtlVector)
		kFocal,					// double
		kRoll,					// double
		kTargetType,			// enum (IAtlSequence::eTargetType)
		kTargetDistance,		// double
		kTargetAngleH,			// double
		kTargetAngleV,			// double
		kClippingBoxMinimum,	// 3D Coordinates (CAtlVector)
		kClippingBoxMaximum,	// 3D Coordinates (CAtlVector)
		kClippingBoxVisible,	// boolean. Not animable in artlantis studio
		kClippingBoxActive,		// boolean
		kDOFUsed,				// boolean
		kDOFType,				// enum (IAtlDepthOfField::EDepthType)
		kDOFClearnessPoint,		// 3D Coordinates (CAtlVector)
		kDOFInvClearnessDist,	// double
		kDOFInvClearnessSize,	// double
		kAtmFlag,				// boolean
		kAtmColor,				// color rvb
		kAtmHighDensity,		// double
		kAtmMediumDensity,		// double
		kAtmLowDensity,			// double
		kAtmScale,				// double
		kAtmMinDistance,		// double
		kAtmMaxDistance,		// double
		kAtmNoise,				// double
		kCamLightFlag,			// boolean
		kCamLightColor,			// color rvb
		kCamLightPower,			// double. Not animable in artlantis studio
		kCamLightRadiosity,		// boolean. Not animable in artlantis studio
		kCamRadiosityLightFlag,	// boolean. Not animable in artlantis studio
		kCamRadiosityLightColor,	// color rvb. Not animable in artlantis studio
		kCamRadiosityLightPower,	// double. Not animable in artlantis studio
		kCamRadiosityLightRadiosity, // boolean. Not animable in artlantis studio

		//kBackgroundColorType,
		kBackgroundColor1, // color rvb
		kBackgroundColor2, // color rvb
		kBackgroundDirectionValue, // double

		// object specific
		kInstancePosition,		// value is a double. the parameter 't' of the bezier spline. 0 is the start of the spline, 1 is the end. 
								/** Warning : the default value of this interpolated value is an absolute 3d coordinate */
		kInstanceRotationX,		// double
		kInstanceRotationY,		// double
		kInstanceRotationZ,		// double
		kInstanceScaleX,		// double
		kInstanceScaleY,		// double
		kInstanceScaleZ,		// double
		kInstanceAttachId,		// UUID (attach Id). Not animable in artlantis studio
		kObjectAnimationBehavior, // enum (IAtlObjectInstance::eAnimationBehavior)
		kObjectTransparency,	// double
		// object - billboard specific
		kInstanceBBOrient,		// boolean
		kInstanceBBHFlip,		// boolean
		kInstanceBBVFlip,		// boolean
		kInstanceBBRenderType,	// enum (IAtlBBObjectInstance::eRenderType)
		kInstanceBBRoughness,	// double
		kInstanceBBShininess,	// double
		kInstanceBBBrightness,	// double
		kInstanceBBContrast,	// double
		// object - trees specific
		kInstance3DPlantColor,				// enum (IAtl3DPlant::eColor)
		kInstance3DPlantDate,				// date : CAtlTime
		kInstance3DPlantHeliodonAttach,		// boolean

		// light specific
		kLightPosition,			// value is a double. the parameter 't' of the bezier spline. 0 is the start of the spline, 1 is the end. 
								/** Warning : the default value of this interpolated value is an absolute 3d coordinate */
		kLightTarget,			// 3D Coordinates (CAtlVector)
		kLightPower,			// double
		kLightType,				// enum (CAtlLight::ELightType)
		kLightFlag,				// boolean
		kLightColor,			// color rvb
		kLightNearDistance,		// double
		kLightFarDistance,		// double
		kLightConeAngle,		// double
		kLightSharpness,		// double
		kLightShadowType,		// enum (CAtlLight::EShadowType)
		kLightShadowPower,		// double
		kLightMapSize,			// integer
		kLightBias,				// double
		kLightRadiosityContribution,	// boolean. Not animable in artlantis studio
		kLightLensFlareFlag,	// boolean
		kLightLensFlareType,	// enum IAtlLensFlare::ELensFlareType
		kLightLensFlareSize,	// double
		kLightLensFlareIntensity,	// double

		// heliodon
		kHeliodonActive,		// boolean. Not animable in artlantis studio
		kHeliodonDate,			// time (CAtlTime)
		kHeliodonPlace,			// Not animable in artlantis studio
		kHeliodonSunshine,		// double
		kHeliodonNorthAngle,	// double. Not animable in artlantis studio
		kHeliodonShadowFlag,	// boolean. Not animable in artlantis studio
		kHeliodonShadowPower,	// double
		kHeliodonShadowBlur,	// integer
		kHeliodonGlobalFlag,	// boolean. Not animable in artlantis studio
		kHeliodonGlobalPower,	// double. Not animable in artlantis studio
		kHeliodonGlobalQuality,	// integer. Not animable in artlantis studio
		kHeliodonLensFlareFlag, // boolean
		kHeliodonLensFlareType,	// enum IAtlLensFlare::ELensFlareType
		kHeliodonLensFlareSize, // double
		kHeliodonLensFlareIntensity, // double
		kHeliodonColoredSunFlag, // boolean. Not animable in artlantis studio
		kHeliodonSunColor1, // color rvb
		kHeliodonSunColor2, // color rvb
		kHeliodonSunColor3, // color rvb
		kHeliodonSunColor4, // color rvb
		kHeliodonSunColor5, // color rvb
		kHeliodonRadiosityContribution, // boolean. Not animable in artlantis studio
		kHeliodonSkylightFlag, // boolean
		kHeliodonSkylightFactor, // double
		kHeliodonSkylightColor, // color rvb
		
		// END OF API 1.2
		// START API 2.0

		// sequence specific
		kTargetObjectUUID,	// UUID
		kClippingBoxPivot,	// 3D Coordinates (CAtlVector)
		kClippingBoxAngle,	// double
		kImageSettingActive, // boolean
		kImageSettingContrast, // double
		kImageSettingBrightness, // double
		kImageSettingSaturation, // double
		kImageSettingMixColorActive, // boolean
		kImageSettingMixColor,		// color rvb
		kExposureActive,			// boolean
		kExposureIntensity,			// double
		kExposureSize,				// double
		kGrainActive,				// boolean
		kGrainBrightness,			// double
		kGrainSize,					// double
		kHatchingActive,			// boolean
		kHatchingThreshold,			// double
		kHatchingDensity,			// double
		kHatchingTransparencyActive,	// boolean
		kHatchingTransparency,		// double

		// object - animated object specific
		kInstanceBBClothesColor,		// integer (interpolated as an enum)
		kInstance3DPlantRenderType,		// enum
		kInstance3DPlantRoughtness,		// double
		kInstance3DPlantShininess,		// double
		kInstance3DPlantBrightness,		// double
		kInstance3DPlantContrast,		// double
		kInstanceAnimatedObjectStart,	// integer
		kInstanceAnimatedPosition,		// double
		kInstancePeopleClothesColor,	// enum

		// light specific
		kLightTargetObjectUUID,			// UUID
		kLightTargetType,				// enum (CAtlLight::eTargetType)

		// mapping node specific
		kAnimatedTexture,				// integer
		kMappingNodeAngle,				// double
		kMappingNodeSizeH,				// double
		kMappingNodeSizeV,				// double
		kMappingNodeSpacingH,			// double
		kMappingNodeSpacingV,			// double
		kMappingNodeRepeatH,			// double
		kMappingNodeRepeatV,			// double
		kMappingNodeRepeatFlagH,		// boolean
		kMappingNodeRepeatFlagV,		// boolean
		kMappingNodeFlipFlagH,			// boolean
		kMappingNodeFlipFlagV,			// boolean
		kMappingNodeMirrorFlagH,		// boolean
		kMappingNodeMirrorFlagV,		// boolean
		kMappingNodeReflection,			// color rvb
		kMappingNodeReflectionLimit,	// double
		kMappingNodeSpecular,			// color rvb
		kMappingNodeGlossy,				// double
		kMappingNodeBump,				// double
		kMappingNodeNormalMap,			// double
		kMappingNodeMixColorFlag,		// boolean
		kMappingNodeMixColor,			// color rvb
		kMappingNodeAlphaFlag,			// boolean
		kMappingNodeTransparencyColorFlag,	// boolean
		kMappingNodeTransparencyColor,		// color rvb
		kMappingNodeCenterX,				// double
		kMappingNodeCenterY,				// double
		kMappingNodeOrigin,					// 3D Coordinates (CAtlVector)
		kMappingNodeNormal,					// 3D Coordinates (CAtlVector)
		kMappingNodeXRefAxis,				// 3D Coordinates (CAtlVector)
		kMappingNodeYRefAxis,				// 3D Coordinates (CAtlVector)
		kMappingNodeBrightness,				// double

		/// property specific
		kPropertyDiffuseColor,				// color rvb
		kPropertyReflectionColor,			// color rvb
		kPropertyReflectionLimit,			// double
		kPropertySpecularColor,				// color rvb
		kPropertyGlossy,					// double
		kPropertyTransparencyColor,			// color rvb
		kPropertyRefractionValue,			// double
		kPropertyNeonColor,					// color rvb
		kPropertyNeonPower,					// double
		kPropertyDistortionSize,			// double
		kPropertyFlatness,					// double
		kPropertyMinReflection,				// double
		kPropertyPowerFactor,				// double
		kPropertyGlazingPropertion,			// double
		kPropertyWindowPanDetection,		// boolean
		kPropertyLightingColor,				// color rvb
		kPropertyLightingPower,				// double
		kPropertyVisibility,				// boolean
		kPropertySmooth,					// double
		kPropertyGlazingWaterSpeed,			// double API v3
		
		// heliodon specific
		kHeliodonCloudCoverage,			// double
		kHeliodonSunshineFlag,			// boolean
		kHeliodonPhi,					// double
		kHeliodonTheta,					// double
		kHeliodonSkylightPowerFlag,		// boolean
		kHeliodonSkylightCustomColorFlag, // boolean
		kHeliodonSkylightMixColorFlag,	// boolean
		kHeliodonSkylightMixColor,		// color rvb
		kHeliodonSkylightPollution,		// double
		kHeliodonCloudsActive,			// boolean
		kHeliodonCloudsCirrusDensity,	// double
		kHeliodonCloudsStratusDensity,	// double
		kHeliodonCloudsCumulusDensity,	// double
		kHeliodonCloudsCirroCumulusDensity,	// double
		kHeliodonFogActive,				// boolean
		kHeliodonFogColor,				// color rvb
		kHeliodonFogDensity,			// double
		kHeliodonFogHeight,				// double
		kHeliodonFogStartDistance,		// double
		kHeliodonFogFocus,				// double
		kHeliodonCloudMaskSun,			// boolean
		kHeliodonCloudsWindDirection,	// double (angle) API 3
		kHeliodonCloudsWindPower,		// double API 3  - new name ? 
		kHeliodonCloudsWindActive,		// boolean API 3
		
		kBackgroundHOffset,				// double
		kBackgroundVOffset,				// double
		kBackgroundColor3				// color rvb
	};
	/// get / set the value type
	virtual eValueType	getValueType() const = 0;
	virtual void		setValueType(eValueType inValue) = 0;

	// get the keyframe list of this interpolated value. for consulting
	virtual const AtlKeyFramesMgr*	getKeysMgr() const = 0;
	// get the keyframe list of this interpolated value. for editing
	virtual AtlKeyFramesMgr*		getKeysMgr() = 0;

	// get the default key frame of this value. It is the state of this value apart from animation.
	virtual const IAtlKeyFrame*		getDefaultKeyFrame() const = 0;
	virtual IAtlKeyFrame*			getDefaultKeyFrame() = 0;

	// return true if the animation of this value is looped
	virtual bool					getLoop() const = 0;
	virtual void					setLoop(bool inLoop) = 0;

	// return number of loops of this value (must be used if "getLoop" returns true).
	virtual int						getNbLoops() const = 0;
	virtual void					setNbLoops(int inLoops) = 0;

	// return the uuid of the used path. uuid is null if the path dosen't exists
	virtual tUUID					getPathId() const = 0;
	virtual void					setPathId(tUUID inId) = 0;
protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual ~IAtlInterpolatedValue() {}
};

}

#endif
